/*  
  TerminPanel.java

  11.03.2001: tk, sourcecode released under the GPL.

  Copyright (c)2001 by SoPra2001 Gruppe 14, University of Dortmund
  mail for contact: thomas.kindler@gmx.de

  This program is free software; you can redistribute it and/or modify it 
  under the terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 2 of the License, or (at your 
  option) any later version. Please read the included "copying.txt" file,
  or the full License at http://www.gnu.org/copyleft for more details.
*/


package gui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import logic.*;

class TerminPanel extends JPanel implements StatusConst {
  BorderLayout borderLayout1 = new BorderLayout();
  JScrollPane jScrollPane1 = new JScrollPane();
  JTable jTable1 = new JTable();
  JPanel jPanel3 = new JPanel();
  BorderLayout borderLayout2 = new BorderLayout();
  JButton jButton_Aendern = new JButton();
  JPanel jPanel2 = new JPanel();
  FlowLayout flowLayout2 = new FlowLayout();
  JButton jButton_Entfernen = new JButton();
  JButton jButton_IncWoche = new JButton();
  JButton jButton_DecWoche = new JButton();
  JPanel jPanel1 = new JPanel();
  JLabel jLabel_Woche = new JLabel();
  FlowLayout flowLayout1 = new FlowLayout();
  DefaultTableModel  terminModel;

  /**
   *  Diese Ansicht braucht Zugriff auf die Nutzerverwaltung
   */
  private Nutzerverwaltung  nutzerVerwaltung;
  private Projekt           projekt;
  private int     anzeigeWoche = 1;

  public TerminPanel( Projekt projekt ) {
    this.nutzerVerwaltung = projekt.getNutzerverwaltung();
    this.projekt = projekt;
    jbInit();
    updateGUI();
  }

  void jbInit() {
    this.setLayout(borderLayout1);
    jTable1.setFont(new java.awt.Font("Dialog", 0, 10));
    jTable1.setToolTipText("Mit einem Doppelklick werden Details angezeigt");
    jTable1.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
    jTable1.setCellSelectionEnabled(true);
    jTable1.setRowHeight(40);
    jTable1.addMouseListener(new java.awt.event.MouseAdapter() {

      public void mouseClicked(MouseEvent e) {
        jTable1_mouseClicked(e);
      }
    });
    jTable1.getTableHeader().setReorderingAllowed(false);
    jTable1.setDefaultRenderer( TextAreaCell.class, new TextAreaRenderer() );
    jTable1.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
    jPanel3.setLayout(borderLayout2);
    jButton_Aendern.setMnemonic('T');
    jButton_Aendern.setText("Termin ndern");
    jButton_Aendern.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        jButton_Aendern_actionPerformed(e);
      }
    });
    jPanel2.setLayout(flowLayout2);
    flowLayout2.setAlignment(FlowLayout.RIGHT);
    jButton_Entfernen.setMnemonic('E');
    jButton_Entfernen.setText("Entfernen");
    jButton_Entfernen.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        jButton_Entfernen_actionPerformed(e);
      }
    });
    jButton_IncWoche.setMnemonic('>');
    jButton_IncWoche.setText(">");
    jButton_IncWoche.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        jButton_IncWoche_actionPerformed(e);
      }
    });
    jButton_DecWoche.setEnabled(false);
    jButton_DecWoche.setMnemonic('<');
    jButton_DecWoche.setText("<");
    jButton_DecWoche.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        jButton_DecWoche_actionPerformed(e);
      }
    });
    jPanel1.setLayout(flowLayout1);
    jLabel_Woche.setText("Woche: 1");
    flowLayout1.setAlignment(FlowLayout.LEFT);
    this.add(jScrollPane1, BorderLayout.CENTER);
    jScrollPane1.getViewport().add(jTable1, null);
    this.add(jPanel3, BorderLayout.SOUTH);
    jPanel3.add(jPanel2, BorderLayout.EAST);
    jPanel2.add(jButton_Aendern, null);
    jPanel2.add(jButton_Entfernen, null);
    jPanel3.add(jPanel1, BorderLayout.WEST);
    jPanel1.add(jButton_DecWoche, null);
    jPanel1.add(jLabel_Woche, null);
    jPanel1.add(jButton_IncWoche, null);
  }

  void jButton_IncWoche_actionPerformed(ActionEvent e) {
    anzeigeWoche++;
    updateGUI();
  }

  void jButton_DecWoche_actionPerformed(ActionEvent e) {
    if (anzeigeWoche > 1) {
      anzeigeWoche--;
      updateGUI();
    }
  }

  void jButton_Entfernen_actionPerformed(ActionEvent e)
  {
    int  row = jTable1.getSelectedRow();
    int  col = jTable1.getSelectedColumn();

    if (row == -1 || col < 1)
      return;

    Nutzer      nutzer = nutzerVerwaltung.getNutzerAt(row);
    Datum       datum  = new Datum( terminModel.getColumnName(col) );
    Stundenplan stdplan = nutzer.getStundenplan();
    Termin      termin  = stdplan.getTermin(datum);

    if (termin == null)
      return;

    JOptionPane.showConfirmDialog(  this,
      "Mchten sie diesen Termin wirklich entfernen?",
      "Termin entfernen", JOptionPane.YES_NO_OPTION
    );

    stdplan.removeTermin(datum);
    updateGUI();
  }


  void jTable1_mouseClicked(MouseEvent e) {
    if (e.getClickCount() >= 2)
      jButton_Aendern_actionPerformed(null);
  }



  void jButton_Aendern_actionPerformed(ActionEvent e) {
    int  row = jTable1.getSelectedRow();
    int  col = jTable1.getSelectedColumn();

    if (row == -1 || col < 1)
      return;

    if (!projekt.getCurrentNutzer().isAdmin())
      return;

    Nutzer      nutzer  = nutzerVerwaltung.getNutzerAt(row);
    Datum       datum   = new Datum( terminModel.getColumnName(col) );
    Stundenplan stdplan = nutzer.getStundenplan();
    Termin      termin  = stdplan.getTermin(datum);

    if (termin == null)
      termin = new Termin();

    TerminDlg dlg = new TerminDlg(this, termin, projekt, true);
    dlg.show();

    if (dlg.userClickedOK()) {
      stdplan.addTermin(datum, termin);
      updateGUI();
    }
  }


  void updateGUI()
  {
    jButton_Aendern.setEnabled( projekt.getCurrentNutzer().isAdmin() );
    jButton_Entfernen.setEnabled( projekt.getCurrentNutzer().isAdmin() );

    jButton_DecWoche.setEnabled(anzeigeWoche > 1);
    jLabel_Woche.setText("Woche: "+anzeigeWoche);

    terminModel = new DefaultTableModel() {
      public boolean isCellEditable(int row, int col) {
        return false;
      }
      public Class getColumnClass(int c) {
        return TextAreaCell.class;
      }
    };

    terminModel.addColumn("Name");

    Datum datum = new Datum(5*(anzeigeWoche-1)+1, false);
    for (int i=0; i<10; i++) {
      terminModel.addColumn(datum.toString());
      datum.next();
    }

    for (int i=0; i<nutzerVerwaltung.getNutzerCount(); i++) {
      Nutzer nutzer = nutzerVerwaltung.getNutzerAt(i);

      Object row[] = new Object[11];
      row[0] = new TextAreaCell(nutzer.getVorname()+"\n"+nutzer.getName());

      datum = new Datum(5*(anzeigeWoche-1)+1, false);
      for (int j=0; j<10; j++) {
        Termin termin = nutzer.getStundenplan().getTermin(datum);
        if (termin != null) {
          TextAreaCell cell = new TextAreaCell(
            termin.getAufgabe().getName() + "\n" + termin.getTaetigkeit()
          );
          switch (termin.getStatus()) {
            case StatusConst.VERZOEGERT:
              cell.setBackground(COLOR_VERZOEGERT); break;
            case StatusConst.GEPLANT:
              cell.setBackground(COLOR_GEPLANT);    break;
          }
          row[j+1] = cell;
        }
        datum.next();
      }
      terminModel.addRow(row);
    }

    jTable1.setModel(terminModel);
  }

}